import pickle
import numpy as np
import platform


def _load_batch(images):
    X = []
    y = []
    for image in images[b'data'][0:5]:
        r = image[:1024].reshape(32, 32)
        g = image[1024:2048].reshape(32, 32)
        b = image[2048:].reshape(32, 32)
        r = np.resize(r, (224, 224))
        g = np.resize(g, (224, 224))
        b = np.resize(b, (224, 224))
        X.append(np.array([r, g, b]))
    for label in images[b'labels'][0:5]:
        label_y = np.zeros(10)
        label_y[label] = 1
        y.append(label_y)
    return X, y


def load(batch_size=25):
    train_X = []
    train_y = []
    base_path = ""
    plat = platform.platform()
    if "Windows" in plat:
        base_path = "F:/cifar-10-batches-py/"
    else:
        base_path = "/Users/weizuo/Downloads/cifar-10-batches-py/"
    for i in range(1, 6):
        images = pickle.load(open(base_path + "data_batch_" + str(i), "rb"),
                             encoding="bytes")
        X, y = _load_batch(images)
        train_X.extend(X)
        train_y.extend(y)

    images = pickle.load(open(base_path + "test_batch", "rb"), encoding="bytes")
    test_X, test_y = _load_batch(images)

    return np.split(np.array(train_X), 25 / batch_size), \
           np.split(np.array(train_y), 25 / batch_size), \
           np.split(np.array(test_X), 25 / batch_size), \
           np.split(np.array(test_y), 25 / batch_size)



import torch
import torch.nn.functional as F
from taolu.vision.vision_units.base_unit import BaseUnits
from taolu.vision.vision_units.losses.yolo_loss import YoloLoss
from taolu.vision.vision_units.losses.mse import MSELoss
from taolu.vision.vision_units.losses.cross_entropy_loss import CrossEntropyLoss
from taolu.vision.vision_units.res_block import ResBlock1, ResBlock2
import taolu.vision.utils.transformers as transformers
import math
import inspect
import json
import os

activation_map = {
    "relu": torch.nn.ReLU(inplace=True),
    "softmax": torch.nn.Softmax()
}

loss_map = {
    "cross_entropy": CrossEntropyLoss,
    "mse": MSELoss,
    "yolo_loss": YoloLoss
}

transformer_map = {
    "cross_entropy_transformer": transformers.cross_entropy_transformer.CrossEntropyTransformer,
    "yolo_loss_transformer": transformers.yolo_transformer.YOLOTransformer,
    "mse_transformer": transformers.mse_transformer.MSETransformer
}


def parse_resblock1(layer, input_shape):
    downsample = layer.get("downsample", False)
    stride = layer.get("stride", 1)
    filters = layer["filters"]
    out_channels = filters
    output_size = int(input_shape[0] / 2) if downsample else input_shape[0]
    output_shape = (output_size, output_size, out_channels)
    unit = BaseUnits(layer['type'])
    resblock = ResBlock1(input_shape[2], out_channels, stride=stride, downsample=downsample)
    unit.seq = resblock
    unit.output_shape = output_shape
    return unit, output_shape


def parse_resblock2(layer, input_shape):
    downsample = layer.get("downsample", False)
    stride = layer.get("stride", 1)
    start_input_channels = layer.get("start_input_channels", int(input_shape[2] / 4))
    output_size = int(input_shape[0] / 2) if downsample else input_shape[0]
    output_shape = (output_size, output_size, start_input_channels * 4)
    unit = BaseUnits(layer['type'])
    resblock = ResBlock2(input_shape[2], start_input_channels, stride=stride, downsample=downsample)
    unit.seq = resblock
    unit.output_shape = output_shape
    return unit, output_shape


def parse_avgpool(layer, input_shape):
    unit = BaseUnits(layer['type'])
    size = layer['size']
    stride = layer.get('stride', 2)
    padding = layer.get("padding", 0)
    output_size = (input_shape[0] - size + 2 * padding) / stride + 1
    output_shape = (output_size, output_size, input_shape[2])
    unit.seq = torch.nn.Sequential(torch.nn.AvgPool2d(size, stride, padding=padding))
    unit.output_shape = output_shape
    return unit, output_shape


def parse_conv2d(layer, input_shape):
    unit_seq = []
    unit = BaseUnits(layer['type'])
    out_channels = layer['filters']
    kernel_size = layer['size']
    stride = layer.get('stride', 1)
    padding = layer.get('padding', 0)
    bias = layer.get("bias", True)
    batch_norm = layer.get('bn', False)
    activation = layer.get('activation', 'relu')
    output_size = (input_shape[0] - kernel_size + 2 * padding) / stride + 1
    output_shape = (output_size, output_size, out_channels)
    unit_seq.append(torch.nn.Conv2d(input_shape[2], out_channels, kernel_size, stride, padding, bias=bias))
    if batch_norm:
        unit_seq.append(torch.nn.BatchNorm2d(output_shape[2]))
    if activation:
        unit_seq.append((activation_map[activation]))
    unit.output_shape = output_shape
    unit.seq = torch.nn.Sequential(*unit_seq)
    return unit, output_shape


def parse_maxpool(layer, input_shape):
    unit = BaseUnits(layer['type'])
    size = layer['size']
    stride = layer.get('stride', 2)
    padding = layer.get("padding", 0)
    output_size = (input_shape[0] - size + 2 * padding) / stride + 1
    output_shape = (output_size, output_size, input_shape[2])
    unit.seq = torch.nn.Sequential(torch.nn.MaxPool2d(size, stride, padding=padding))
    unit.output_shape = output_shape
    return unit, output_shape


def parse_globalavgpool(layer, input_shape):
    unit = BaseUnits(layer['type'])
    unit.seq = torch.nn.Sequential(torch.nn.AvgPool2d(input_shape[0]))
    output_shape = (1, 1, input_shape[2])
    unit.output_shape = output_shape
    return unit, output_shape


def parse_linear(layer, input_shape):
    unit_seq = []
    unit = BaseUnits(layer['type'])
    output = layer['output']
    activation = layer.get('activation', 'relu')
    output_shape = (1, 1, output)
    unit_seq.append(torch.nn.Linear(input_shape[2] * input_shape[0] * input_shape[1], output))
    unit.output_shape = output_shape
    if activation:
        unit_seq.append((activation_map[activation]))
    unit.seq = torch.nn.Sequential(*unit_seq)
    return unit, output_shape


def parse_dropout(layer, input_shape):
    unit = BaseUnits(layer['type'])
    rate = layer.get("rate", 0.5)
    unit.seq = torch.nn.Sequential(torch.nn.Dropout2d(rate))
    unit.output_shape = input_shape
    return unit, input_shape


def parse_optim(optim_cfg, parameters):
    optim_type = optim_cfg.get('optim', 'adam')
    optim_type = optim_type.lower()
    optim = list(filter(lambda x: type(x) is tuple and inspect.isclass(x[1]) and x[1].__name__.lower() == optim_type,
                        inspect.getmembers(torch.optim)))
    if len(optim) > 0:
        return optim[0][1](parameters)
    return torch.optim.Adam(parameters)


def parse_backbone(backbone_cfg, input_shape):
    backbone_net_name = backbone_cfg['name']
    backbone_origin_cfg = json.load(open(os.path.join(os.path.dirname(__file__), backbone_net_name + ".json")))
    backbone_origin_layers_cfg = backbone_origin_cfg['layers']
    remove_indexes = backbone_cfg.get("remove", [0, 0])
    remove_indexes[0] = len(backbone_origin_layers_cfg) + remove_indexes[0] if remove_indexes[0] < 0 else \
        remove_indexes[0]
    remove_indexes[1] = len(backbone_origin_layers_cfg) + remove_indexes[1] if remove_indexes[1] < 0 else \
        remove_indexes[1]
    remove_indexes = list(sorted(remove_indexes))

    del backbone_origin_layers_cfg[remove_indexes[0]:remove_indexes[1] + 1]
    seq, output_shape = parse_layers(backbone_origin_layers_cfg, input_shape)
    unit = BaseUnits(backbone_cfg['type'])
    unit.seq = torch.nn.Sequential(*seq)
    unit.output_shape = output_shape
    return unit, output_shape


def parse_layers(layers_cfg, output_shape):
    seq = []
    for layer in layers_cfg:
        layer_type = layer['type']
        unit, output_shape = eval("parse_" + layer_type + "(layer,output_shape)")
        seq.append(unit)
    return seq, seq[-1].output_shape


class BaseNet(torch.nn.Module):
    def __init__(self, cfg):
        self.loss = None
        super(BaseNet, self).__init__()
        self.seq = []
        in_channels = cfg.get("channels", 3)
        width = cfg['width']
        height = cfg['height']
        output_shape = (height, width, in_channels)
        seq, output_shape = parse_layers(cfg['layers'], output_shape)
        self.seq.extend(seq)
        loss_name = cfg.get("loss", "mse")
        self.loss = loss_map[loss_name](cfg)
        self.seq = torch.nn.Sequential(*self.seq)
        self.optim = parse_optim(cfg, self.seq.parameters())
        self.transformer = transformer_map[loss_name + "_transformer"](cfg)
        self.init_weights()

    def init_weights(self):
        for m in self.seq.modules():
            if isinstance(m, torch.nn.Conv2d):
                n = m.kernel_size[0] * m.kernel_size[1] * m.out_channels
                m.weight.data.normal_(0, math.sqrt(2. / n))
            elif isinstance(m, torch.nn.BatchNorm2d):
                m.weight.data.fill_(1)
                m.bias.data.zero_()

    def forward(self, x):
        last_unit = None
        for i, unit in enumerate(self.seq.children()):
            if "linear" in unit.type:
                if i > 0 and last_unit.type != "linear":
                    x = x.view(x.size(0), -1)
            x = unit(x)
            last_unit = unit
        x = torch.nn.functional.softmax(x, 1)
        return x

    def cuda(self, device=None):
        self.seq.cuda(device)
        return self

    def cpu(self):
        self.seq.cpu()
        return self



from .loss import Loss
from torch.nn.functional import cross_entropy


class CrossEntropyLoss(Loss):
    def __init__(self, cfg):
        super(CrossEntropyLoss, self).__init__(cfg)

    def forward(self, pred, target):
        return cross_entropy(pred, target)



from .loss import Loss
from torch.nn.functional import mse_loss


class MSELoss(Loss):
    def __init__(self, cfg):
        super(MSELoss, self).__init__(cfg)

    def forward(self, pred, target):
        return mse_loss(pred, target)



import torch
from taolu.vision.base_nets.base_net import BaseNet


class ClassificationJob:
    def __init__(self, cfg, num_classes=None, cuda=False):
        self.num_classes = num_classes
        self.cfg = cfg
        self.cuda = cuda
        self.model = self.build_model()

    def build_model(self):
        if self.num_classes is not None:
            self.cfg.layers[-1].output = self.num_classes
        model = BaseNet(self.cfg)
        return model.cuda() if self.cuda else model

    def train(self, x, y):

        # x = torch.Tensor(x)
        # y = torch.Tensor(y)
        if self.model.transformer:
            x, y = self.model.transformer.transform(x, y)
        if self.cuda:
            x = x.cuda()
            y = y.cuda()
        output = self.model(x)
        error = self.model.loss(output, y)
        self.model.optim.zero_grad()
        error.backward()
        self.model.optim.step()
        return error



from taolu.vision.utils.transformers.transformer import Transformer
import torch


class CrossEntropyTransformer(Transformer):
    def __init__(self, cfg):
        super(CrossEntropyTransformer, self).__init__(cfg)

    def transform(self, images, labels):
        images = torch.Tensor(images)
        labels = torch.Tensor(labels)
        labels = labels.to(dtype=torch.long)
        return images, torch.argmax(labels, 1)



import torch
import json
from taolu.vision.jobs.classification import ClassificationJob
from taolu.demo.demo_base import DemoBase

if __name__ == '__main__':
    demo = DemoBase("../vision/base_nets/resnet18.json", "classification")
    demo.go()



import torch
import torch.nn.functional as F
from .loss import Loss


class YoloLoss(Loss):
    def __init__(self, cfg):
        super(YoloLoss, self).__init__(cfg)
        self.S = cfg['C']
        self.B = cfg['B']
        self.classes = cfg['num_classes']
        self.lambda_coobj = cfg['lambda_coobj']
        self.lambda_noobj = cfg['lambda_noobj']

    def compute_iou(self, box1, box2):
        # N==2, M==1?
        N = box1.size(0)
        M = box2.size(0)

        # choose the left top
        lt = torch.max(
            box1[:, :2].unsqueeze(1).expand(N, M, 2),  # [N,2] -> [N,1,2] -> [N,M,2]
            box2[:, :2].unsqueeze(0).expand(N, M, 2),  # [M,2] -> [1,M,2] -> [N,M,2]
        )
        rb = torch.min(
            box1[:, 2:].unsqueeze(1).expand(N, M, 2),  # [N,2] -> [N,1,2] -> [N,M,2]
            box2[:, 2:].unsqueeze(0).expand(N, M, 2),  # [M,2] -> [1,M,2] -> [N,M,2]
        )

        wh = rb - lt  # [N,M,2]
        wh[wh < 0] = 0  # clip at 0

        # The area of the intersect zone
        inter = wh[:, :, 0] * wh[:, :, 1]  # [N,M]

        # The area of box1
        area1 = (box1[:, 2] - box1[:, 0]) * (box1[:, 3] - box1[:, 1])  # [N,]
        # The area of box2
        area2 = (box2[:, 2] - box2[:, 0]) * (box2[:, 3] - box2[:, 1])  # [M,]
        area1 = area1.unsqueeze(1).expand_as(inter)  # [N,] -> [N,1] -> [N,M]
        area2 = area2.unsqueeze(0).expand_as(inter)  # [M,] -> [1,M] -> [N,M]
        iou = inter / (area1 + area2 - inter)
        return iou

    def forward(self, pred, gt):
        '''
        The ground truth boxes only penalize the corresponding pred cells (has obj penal).
        The ground truth no boxes only penalize the corresponding pred cells (has no obj penal)
        '''

        N = pred.size()[0]
        batch_size = pred.size()[0]
        pred = pred.view(N, self.S, self.S, self.B * 5 + self.classes)
        gt_has_obj_factors = (gt[:, :, :, 4] > 0).unsqueeze(-1).expand_as(gt)  # (batch,h,w, B*2+C)
        gt_no_obj_factors = (gt[:, :, :, 4] == 0).unsqueeze(-1).expand_as(gt)

        # This extracts all the cells corresponding with the gt has object cells. Also the batch is mixed in.
        has_obj_pred = pred[gt_has_obj_factors].view(-1, self.B * 5 + self.classes)  # has obj pred ()

        # Only obtain the x,y,w,h,confidence, # all the pred boxes corresponding with the ground truth boxes, batch?
        boxes_pred = has_obj_pred[:, :self.B * 5].contiguous().view(-1, 5)
        # only obtain the Classespred.size()
        classes_pred = has_obj_pred[:, self.B * 5:]

        has_obj_gt = gt[gt_has_obj_factors].view(-1, self.B * 5 + self.classes)

        # all the ground truth boxes also with the emply bounding boxes (x,y,w,h,C)
        boxes_gt = has_obj_gt[:, :self.B * 5].contiguous().view(-1, 5)
        classes_gt = has_obj_gt[:, self.B * 5:]

        # The last item in the yolo loss
        class_loss = F.mse_loss(classes_pred, classes_gt, size_average=False)

        # the pred boxes corresponding with the ground truth no object boxes.
        no_obj_pred = pred[gt_no_obj_factors].view(-1, self.B * 5 + self.classes)
        no_obj_gt = gt[gt_no_obj_factors].view(-1, self.B * 5 + self.classes)
        no_obj_multiplier = torch.zeros(no_obj_pred.size(), dtype=torch.uint8, device=no_obj_pred.device)

        # When the confidence is 1
        no_obj_multiplier[:, 4] = 1
        no_obj_multiplier[:, 9] = 1

        # Only extracts the pred cells confidences  which correspond with gt no obj boxes
        no_obj_pred_C = no_obj_pred[no_obj_multiplier]

        # Only extracts the pred cells confidences which correspond with gt no obj boxes
        no_obj_gt_C = no_obj_gt[no_obj_multiplier]

        # Penalize the no object Confidence
        noo_obj_C_loss = F.mse_loss(no_obj_pred_C, no_obj_gt_C, size_average=False)

        has_obj_resp_multiplier = torch.zeros(boxes_gt.size(), dtype=torch.uint8, device=boxes_gt.device)
        has_obj_no_resp_multiplier = torch.zeros(boxes_gt.size(), dtype=torch.uint8, device=boxes_gt.device)
        for i in range(0, boxes_gt.size()[0], self.B):
            # choose the corresponding pred cell two boxes
            pred_box = boxes_pred[i:i + self.B]
            pred_box_coords = torch.autograd.Variable(torch.FloatTensor(pred_box.size()))

            # normalize the coords, (x,y,w,h) -> (left_top_x, left_top_y, right_bottom_x, right_bottom_y)
            pred_box_coords[:, :2] = pred_box[:, :2] - 0.5 * pred_box[:, 2:4]
            pred_box_coords[:, 2:4] = pred_box[:, :2] + 0.5 * pred_box[:, 2:4]
            gt_box = boxes_gt[i].view(-1, 5)

            gt_box_coords = torch.autograd.Variable(torch.FloatTensor(gt_box.size()))
            gt_box_coords[:, :2] = gt_box[:, :2] - 0.5 * gt_box[:, 2:4]
            gt_box_coords[:, 2:4] = gt_box[:, :2] + 0.5 * gt_box[:, 2:4]

            # compute the iou of two boxes of pred and one boxes of gt to check which box of the pred is choosed to response to the gt
            # the
            iou = self.compute_iou(pred_box_coords[:, :4], gt_box_coords[:, :4])
            max_iou, max_index = iou.max(0)
            max_index = max_index.data
            has_obj_resp_multiplier[i + max_index] = 1
            has_obj_no_resp_multiplier[i + 1 - max_index] = 1

        boxes_pred_response = boxes_pred[has_obj_resp_multiplier].view(-1, 5)
        boxes_gt_response = boxes_gt[has_obj_resp_multiplier].view(-1, 5)
        contain_loss = F.mse_loss(boxes_pred_response[:, 4], boxes_gt_response[:, 4], size_average=False)
        loc_loss = F.mse_loss(boxes_pred_response[:, :2], boxes_gt_response[:, :2], size_average=False) + F.mse_loss(
            boxes_pred_response[:, 2:4], boxes_gt_response[:, 2:4], size_average=False)

        boxes_pred_not_response = boxes_pred[has_obj_no_resp_multiplier].view(-1, 5)
        boxes_gt_not_response = boxes_gt[has_obj_no_resp_multiplier].view(-1, 5)
        boxes_gt_not_response[:, 4] = 0
        not_contain_loss = F.mse_loss(boxes_pred_not_response[:, 4], boxes_gt_not_response[:, 4], size_average=False)

        return (
                       self.lambda_coobj * loc_loss + contain_loss + not_contain_loss + self.lambda_noobj * noo_obj_C_loss + class_loss) / N



from taolu.vision.vision_units.losses.yolo_loss import YoloLoss
import torch
import taolu.demo.demo_images
import numpy as np
import cv2
from taolu.demo.yolo_temp import YOLO
from taolu.vision.vision_units.losses.yolo_loss import YoloLoss


def encoder(boxes, labels):
    '''
    boxes (tensor) [[x1,y1,x2,y2],[]]
    labels (tensor) [...]
    return 7x7x30
    '''
    # boxes = torch.Tensor(boxes)
    # labels = torch.Tensor(labels)
    target = np.zeros((7, 7, 30))
    cell_size = 1. / 7 * 224
    wh = boxes[:, 2:] - boxes[:, :2]
    cxcy = (boxes[:, 2:] + boxes[:, :2]) / 2
    for i in range(cxcy.shape[0]):
        cxcy_sample = cxcy[i]
        ij = np.ceil(cxcy_sample / cell_size) - 1  #
        target[int(ij[1]), int(ij[0]), 4] = 1
        target[int(ij[1]), int(ij[0]), 9] = 1
        target[int(ij[1]), int(ij[0]), int(labels[i]) + 9] = 1
        xy = ij * cell_size  # 匹配到的网格的左上角相对坐标
        delta_xy = (cxcy_sample - xy) / cell_size
        target[int(ij[1]), int(ij[0]), 2:4] = wh[i]
        target[int(ij[1]), int(ij[0]), :2] = delta_xy
        target[int(ij[1]), int(ij[0]), 7:9] = wh[i]
        target[int(ij[1]), int(ij[0]), 5:7] = delta_xy
    return target


def translate_image(images, boxes):
    new_images = []
    new_boxes = []
    images[0].transpose(2, 0, 1)
    for i in range(len(images)):
        image = cv2.resize(images[i], (224, 224))
        box = boxes[i]
        if box.shape[0] == 0:
            continue
        new_images.append(image.transpose(2, 0, 1))
        box[:, 0] = box[:, 0] - 80
        box[:, 1:] = box[:, 1:] / 640 * 224
        new_boxes.append(encoder(box[:, 1:], box[:, 0]))

    return new_images, torch.Tensor(new_boxes)


if __name__ == '__main__':
    train_data = np.load("demo_images/obj_detect_data.npz")
    loss_fun = YoloLoss()
    images, boxes = translate_image(train_data['images'], train_data['boxes'])
    cuda_available = torch.cuda.is_available()
    yolo = YOLO()
    if cuda_available:
        yolo.cuda()
    optimizer = torch.optim.Adam(yolo.parameters())
    for i in range(100):
        input = torch.autograd.Variable(torch.Tensor(images[0:2]))
        label = torch.autograd.Variable(torch.Tensor(boxes[0:2]))
        if cuda_available:
            input = input.cuda()
            label = label.cuda()
        pred = yolo(input)
        error = loss_fun(pred, label)
        optimizer.zero_grad()
        error.backward()
        optimizer.step()
        print(error)



import torch
import json
from taolu.vision.jobs.classification import ClassificationJob
from taolu.vision.jobs.object_detection import ObjectDetectionJob
from taolu.vision.utils.cal101_util import load
import numpy as np


class DemoBase:
    def __init__(self, cfg_file, job_type):
        self.cfg = json.load(open(cfg_file))
        self.job_type = job_type
        cuda_available = torch.cuda.is_available()
        if job_type == "classification":
            self.job = ClassificationJob(self.cfg, cuda=cuda_available)
            self.train_X, self.train_y = load((self.job.cfg["width"], self.job.cfg['height']))
        else:
            self.job = ObjectDetectionJob(self.cfg, cuda=cuda_available)
            train_data = np.load("demo_images/obj_detect_data.npz")
            self.train_X = train_data['images'][0:2]
            self.train_y = train_data['boxes'][0:2]

    def go(self):
        for epoch in range(100):
            error = self.job.train(self.train_X, self.train_y)
            print(error)



import os
import json

# nets = {}
# base_dir = os.path.dirname(__file__)
# files = os.listdir(base_dir)
# files = list(filter(lambda x: ".json" in x))
# for file in files:
#     cfg = json.load(open(os.path.join(base_dir, file)))



import torch
from .transformer import Transformer
import cv2
import numpy as np


class YOLOTransformer(Transformer):
    def __init__(self, cfg):
        super(YOLOTransformer, self).__init__(cfg)

    def transform(self, images, boxes):
        new_images = []
        new_boxes = []
        for i in range(len(images)):
            origin_size = [images[i].shape[0], images[i].shape[1]]
            image = cv2.resize(images[i], (self.cfg['height'], self.cfg['width']))
            box = boxes[i]
            if box.shape[0] == 0:
                continue
            if image.shape[0] != 3:
                image = image.transpose(2, 0, 1)
            new_images.append(image)
            box[:, 1] = box[:, 1] / origin_size[1] * self.cfg['width']
            box[:, 2] = box[:, 2] / origin_size[0] * self.cfg['height']
            box[:, 3] = box[:, 3] / origin_size[1] * self.cfg['width']
            box[:, 4] = box[:, 4] / origin_size[0] * self.cfg['height']
            new_boxes.append(self.encoder(box[:, 1:], box[:, 0]))
        return torch.Tensor(new_images), torch.Tensor(new_boxes)

    def encoder(self, boxes, labels):
        '''
        boxes (tensor) [[x1,y1,x2,y2],[]]
        labels (tensor) [...]
        return 7x7x30
        '''
        target = np.zeros((self.cfg['C'], self.cfg['C'], self.cfg['B'] * 5 + self.cfg['num_classes']))
        cell_size = 1. / self.cfg['C'] * self.cfg['height']
        wh = boxes[:, 2:] - boxes[:, :2]
        cxcy = (boxes[:, 2:] + boxes[:, :2]) / 2
        for i in range(cxcy.shape[0]):
            cxcy_sample = cxcy[i]
            ij = np.ceil(cxcy_sample / cell_size) - 1  #
            for b in range(self.cfg['B']):
                target[int(ij[1]), int(ij[0]), b * 5 + 4] = 1
            target[int(ij[1]), int(ij[0]), int(labels[i]) + self.cfg['B'] * 5] = 1
            xy = ij * cell_size
            delta_xy = (cxcy_sample - xy) / cell_size
            target[int(ij[1]), int(ij[0]), 2:4] = wh[i]
            target[int(ij[1]), int(ij[0]), :2] = delta_xy
            target[int(ij[1]), int(ij[0]), 7:9] = wh[i]
            target[int(ij[1]), int(ij[0]), 5:7] = delta_xy
        return target



from torchvision.models import vgg16
import torch.nn as nn
import torch


class YOLO(torch.nn.Module):
    def __init__(self):
        super(YOLO, self).__init__()
        self.model = vgg16(True)

        # model = vgg16()
        self.model.classifier = nn.Sequential(
            nn.Linear(512 * 7 * 7, 4096),
            nn.ReLU(True),
            nn.Dropout(),
            nn.Linear(4096, 4096),
            nn.ReLU(True),
            nn.Dropout(),
            nn.Linear(4096, 1470),
        )

    def forward(self, x):
        x = self.model.features(x)
        x = x.view(x.size(0), -1)
        x = self.model.classifier(x)
        x = x.view(-1, 7, 7, 30)
        return x



import torch


class ResBlock1(torch.nn.Module):
    def __init__(self, input_channels, output_channels, stride=1, downsample=False):
        super(ResBlock1, self).__init__()
        self.conv1 = torch.nn.Conv2d(input_channels, output_channels, 3, bias=False, padding=1, stride=stride)
        self.bn1 = torch.nn.BatchNorm2d(output_channels)
        self.relu = torch.nn.ReLU(inplace=True)
        self.conv2 = torch.nn.Conv2d(output_channels, output_channels, 3, bias=False, padding=1)
        self.bn2 = torch.nn.BatchNorm2d(output_channels)
        self.downsample = torch.nn.Sequential(
            torch.nn.Conv2d(input_channels, output_channels, kernel_size=1, stride=stride, bias=False),
            torch.nn.BatchNorm2d(output_channels)) if downsample else None
        self.stride = stride

    def forward(self, x):
        residual = x
        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)
        out = self.conv2(out)
        out = self.bn2(out)
        if self.downsample is not None:
            residual = self.downsample(x)
        out += residual
        out = self.relu(out)
        return out


class ResBlock2(torch.nn.Module):
    def __init__(self, input_channels, start_input_channels, stride=1, downsample=False):
        super(ResBlock2, self).__init__()
        self.conv1 = torch.nn.Conv2d(input_channels, start_input_channels, kernel_size=1, bias=False)
        self.bn1 = torch.nn.BatchNorm2d(start_input_channels)
        self.conv2 = torch.nn.Conv2d(start_input_channels, start_input_channels, kernel_size=3, stride=stride,
                                     padding=1, bias=False)
        self.bn2 = torch.nn.BatchNorm2d(start_input_channels)
        self.conv3 = torch.nn.Conv2d(start_input_channels, start_input_channels * 4, kernel_size=1, bias=False)
        self.bn3 = torch.nn.BatchNorm2d(start_input_channels * 4)
        self.relu = torch.nn.ReLU(inplace=True)
        self.downsample = torch.nn.Sequential(
            torch.nn.Conv2d(input_channels, start_input_channels * 4, kernel_size=1, stride=stride, bias=False),
            torch.nn.BatchNorm2d(start_input_channels * 4)) if downsample else None
        self.stride = stride

    def forward(self, x):
        residual = x
        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)
        out = self.conv2(out)
        out = self.bn2(out)
        out = self.relu(out)
        out = self.conv3(out)
        out = self.bn3(out)
        if self.downsample is not None:
            residual = self.downsample(x)
        out += residual
        out = self.relu(out)
        return out



from setuptools import setup, find_packages

setup(
    name='taolunet',
    version='0.1.2',
    author='Wei ZUO',
    author_email='victorzuo@outlook.com',
    packages=find_packages(),
    url='https://github.com/ainrichman/taolunet',
    include_package_data=True,
    license='LICENSE.txt',
    description='Taolunet on Pytorch.',
    long_description=open('README.txt').read(),
    install_requires=[
        "torch==0.4.0",
        "numpy==1.14.5",
        "torchvision==0.2.1",
        "opencv_python==3.3.0.10",
        "scipy==1.0.0"
    ],
)



from taolu.vision.utils.transformers.transformer import Transformer
import torch


class MSETransformer(Transformer):
    def __init__(self, cfg):
        super(MSETransformer, self).__init__(cfg)

    def transform(self, images, labels):
        return torch.Tensor(images), torch.Tensor(labels)



import torch


class BaseUnits(torch.nn.Module):
    def __init__(self, type):
        super(BaseUnits, self).__init__()
        self.type = type
        self.seq = torch.nn.Sequential()
        self.output_shape = ()

    def forward(self, x):
        return self.seq(x)



import numpy as np
import platform
import os
from scipy import misc


def load(target_size):
    X = []
    y = []
    plat = platform.platform()
    if "Windows" in plat:
        base_path = "F:/101_ObjectCategories/"
    else:
        base_path = "/Users/weizuo/Downloads/101_ObjectCategories/"
    labels = {}
    cates = os.listdir(base_path)
    for cate in cates[0:2]:
        if ".DS_Store" in cate:
            continue
        if cate in labels:
            index = labels[cate]
        else:
            index = len(labels)
            labels[cate] = index
        label_y = np.zeros(101)
        label_y[index] = 1
        images_dir = os.path.join(base_path, cate)
        for image_file in os.listdir(images_dir):
            image_data = misc.imread(os.path.join(images_dir, image_file))
            image_data = np.resize(image_data, (3, target_size[0],target_size[1]))
            X.append(image_data)
            y.append(label_y)
    return np.array(X[0:6]), np.array(y[0:6])
